@@ -3,9 +3,9 @@ use strict;
 use Carp qw( croak );
 
 use vars qw( $VERSION );
-$VERSION = 0.16;
+$VERSION = '0.19';
 
-use vars qw( %forbidden );
+use vars qw(%forbidden);
 
 sub get_forbidden_list {
   \%forbidden
@@ -17,7 +17,8 @@ sub import {
   my $forbidden = get_forbidden_list;
   
   for (@forbidden_modules) {
-      $forbidden->{$_} = $INC{ module2file($_) };
+      my $file = module2file($_);
+      $forbidden->{$file} = delete $INC{$file};
   };
 
   # Scrub %INC, so that loaded modules disappear
@@ -25,21 +26,16 @@ sub import {
     scrub( $module );
   };
 
-  unshift @INC, \&fake_module;
+  @INC = (\&fake_module, grep { !ref || $_ != \&fake_module } @INC);
 };
 
 sub fake_module {
     my ($self,$module_file,$member_only) = @_;
     # Don't touch $@, or .al files will not load anymore????
 
-    my $forbidden = get_forbidden_list;
-
-    my $modulename = file2module($module_file);
-
-    # Deliver a faked, nonworking module
-    if (exists $forbidden->{$modulename}) {
-      my @faked_module = ("package $modulename;","0;");
-      return sub { defined ( $_ = shift @faked_module ) };
+    if (exists get_forbidden_list->{$module_file}) {
+      my $module_name = file2module($module_file);
+      croak "Can't locate $module_file in \@INC (you may need to install the $module_name module) (\@INC contains: @INC)";
     };
 };
 
@@ -49,12 +45,12 @@ sub unimport {
   my $forbidden = get_forbidden_list;
 
   for $module (@list) {
-    if (exists $forbidden->{$module}) {
-      if (defined $forbidden->{$module}) {
-          $INC{ module2file($module) } = delete $forbidden->{$module};
-      } else {
-          delete $forbidden->{$module};
-      };
+    my $file = module2file($module);
+    if (exists $forbidden->{$file}) {
+      my $path = delete $forbidden->{$file};
+      if (defined $path) {
+        $INC{ $file } = $path;
+      }
     } else {
       croak "Can't allow non-forbidden module $module";
     };
@@ -77,12 +73,7 @@ sub module2file {
 
 sub scrub {
   my ($module) = @_;
-  for my $key (keys %INC) {
-    my $fn = file2module($key);
-    if ($fn =~ /\Q$module\E$/) {
-        delete $INC{$key};
-    };
-  };
+  delete $INC{module2file($module)};
 };
 
 1;
@@ -130,7 +121,7 @@ if you are testing and/or debugging this module.
 =head1 ONE LINER
 
 A neat trick for using this module from the command line
-was mentioned to me by NUFFIN:
+was mentioned to me by NUFFIN and by Jerrad Pierce:
 
   perl -MTest::Without::Module=Some::Module -w -Iblib/lib t/SomeModule.t
 
@@ -141,7 +132,9 @@ when a certain module is unavailable.
 
 =over 4
 
-=item * There is no lexicalic scoping
+=item *
+
+There is no lexical scoping
 
 =back
 
@@ -149,7 +142,7 @@ when a certain module is unavailable.
 
 Much improvement must be thanked to Aristotle from PerlMonks, he pointed me
 to a much less convoluted way to fake a module at
-L<http://www.perlmonks.org/index.pl?node=192635>.
+L<https://perlmonks.org?node=192635>.
 
 I also discussed with him an even more elegant way of overriding
 CORE::GLOBAL::require, but the parsing of the overridden subroutine
@@ -163,12 +156,28 @@ of a module.
 
 =head1 AUTHOR
 
-Copyright (c) 2003-2007 Max Maischein, E<lt>corion@cpan.orgE<gt>
+Copyright (c) 2003-2014 Max Maischein, E<lt>corion@cpan.orgE<gt>
 
 =head1 LICENSE
 
 This module is released under the same terms as Perl itself.
 
+=head1 REPOSITORY
+
+The public repository of this module is
+L<https://github.com/Corion/test-without-module>.
+
+=head1 SUPPORT
+
+The public support forum of this module is
+L<https://perlmonks.org/>.
+
+=head1 BUG TRACKER
+
+Please report bugs in this module via the RT CPAN bug queue at
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=Test-Without-Module>
+or via mail to L<test-without-module-Bugs@rt.cpan.org>.
+
 =head1 SEE ALSO
 
 L<Devel::Hide>, L<Acme::Intraweb>, L<PAR>, L<perlfunc>
